<?php
/*
ftsutils.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
VERSION: '$Id: ftsutils.php,v 1.18 2010/05/19 13:36:53 schwarz Exp $';
AUTHOR: Lionel Schwarz (lionel.schwarz@in2p3.fr)
*/

function ftsSyslog($msg, $level=LOG_NOTICE) {
  global $ftsdebug;
  if ($ftsdebug or $level < LOG_NOTICE) syslog($level,$msg);
}

function ftsCacheReadQuery($query, $refresh) {
  return ftsCacheRead(md5($query), $refresh);
}

function ftsCacheWriteQuery($query, $obj) {
  return ftsCacheWrite(md5($query), $obj);
}

function ftsCacheRead($cachefile, $refresh) {
  if ($refresh < 0) throw new Exception('CACHE_NOT_REQUESTED');
  $cache = 'cache/' . $cachefile;
  $expire = time()-$refresh;

  if (file_exists($cache) && filemtime($cache)>$expire) {
    return unserialize(file_get_contents($cache));
  }
  throw new Exception('NOT_IN_CACHE');
}

function ftsCacheWrite($cachefile, $obj) {
  $cache = 'cache/' . $cachefile;
  $serialized = serialize($obj);
  file_put_contents($cache, $serialized);
}

function ftsCacheClear($refresh=86400) {
  $cachefile = 'cache/clear';
  $expire = time()-$refresh;
  if (file_exists($cachefile) && filemtime($cachefile)>$expire) return;
  foreach (scandir('cache') as $f) {
    $filename = 'cache/' . $f;
    if (is_file($filename) && filemtime($filename)<$expire && $f != $cachefile) {
      unlink($filename);
      ftsSyslog("Removed cache $f");
    }
  }
  touch($cachefile);
}

function getValidSurl($surl) {
  if (trim($surl) == '') return trim($surl);
  // si srm://xxx.xxx.xxx.xxx:xxxx/srm/managervX?SFN=/...
  //if (preg_match("/srm:\/\/", $surl)==1)  return $surl;
  $surlElements = parse_url($surl);
  //print_r($surlElements);
  if ($surlElements) {
    if ($surlElements['scheme'] != 'srm')
      throw new InvalidInputException("Not a valid SURL: $surl");
    if ($surlElements['path'] == '/srm/managerv1' || $surlElements['path'] == '/srm/managerv2') {
      if (substr( $surlElements['query'] ,0,4) != 'SFN=')
        throw new InvalidInputException("Not a valid SURL: $surl");
      return $surl;
    } else { // Dans le cas srm://xxx:xxx/pnfs/...
      $newsurl = $surlElements['scheme'] . '://';
      $newsurl .= $surlElements['host'];
      if ($surlElements['port'] != '')
        $newsurl .= ':' . $surlElements['port'];
      else
        $newsurl .= ':8443';
      $newsurl .= '/srm/managerv2?SFN=';
      $newsurl .= $surlElements['path'];
      //return $newsurl;
      return trim($surl);
    }
  }
  throw new InvalidInputException("Not a valid SURL: $surl");
}

function isValidJobId($jobid) {
  if (trim($jobid) == '') return true;

  // Check job id like: 1b159557-d627-11dc-bcb5-fd68ca48d132
  return ereg ("([[:xdigit:]]{8})-([[:xdigit:]]{4})-([[:xdigit:]]{4})-([[:xdigit:]]{4})-([[:xdigit:]]{12})", $jobid);
}

function getHumanSize($size_in_bytes) {
    $debit = $size_in_bytes +0;
    $units=Array("B", "KB", "MB", "GB");
    $indice=0;
    while ($debit >= 1024) {
      $debit = $debit/1024.0;
      $indice++;
    }
    return sprintf("%.2f %s", $debit, $units[$indice]);
}

/*
function isValidState($st) {
  global $states;
  return in_array($st, $states);
}
*/
function isValidChannel($channel) {
  if ($channel === FTS_NONE or $channel === FTS_ALL) return True;
  return ereg ("([[:alnum:]]+)-([[:alnum:]]+)", $channel);
}

function isValidFileid($fileid) {
  return (preg_match('/^[0-9]+$/', $fileid) > 0);
}
/*
function isValidTimescale($t) {
  global $conf;
  return in_array($t, $conf->timescalelist);
}

function validateTimescale($t) {
  global $conf;
  if (! is_numeric($t)) {
    return $conf->defaulttimescale;
  } else if (! isValidTimescale($t)) {
    return $conf->defaulttimescale;
  } else {
    return $t;
  }
}
*/
class ChannelStat {
  var $name;
  var $nb_jobs;

  function __construct($name) {
	global $ftsm;
        $this->name = $name;
        foreach ($ftsm->getStates() as $state) {
          $this->setStat($state);
	}
  }

  function setStat($state, $nb = 0) {
        $this->nb_jobs[$state] = $nb;
  }

  public function hasErrors() {
    return ($this->nb_jobs[FTS_STATE_FINISHED_DIRTY] + $this->nb_jobs[FTS_STATE_FAILED] > 0);
  }

  function toString() {
        global $ftsm;
        $nbtot=0;
        foreach ($this->nb_jobs as $nb) {
          $nbtot += $nb;
        }
        $sizetot=500;
        $res = "";
        //print "<div>";
        foreach ($ftsm->getStates() as $state) {
          if ($this->nb_jobs[$state] != 0) {
            //$res .= sprintf("<img title=\"%d %s\" src=\"%s.gif\" width=\"%d\" height=\"10\">", $this->nb_jobs[$state], $state, $state, (int)(($this->nb_jobs[$state]*$sizetot)/$nbtot));
            $perc=($this->nb_jobs[$state]*100)/$nbtot;
            if ($perc >= 10) {
              $res .= sprintf("<div class=\"%s\" style=\"width: %d%%\" >%d</div>", 
                $state,
                $perc,
                $this->nb_jobs[$state]);
            } else {
              $res .= sprintf("<div class=\"%s\" style=\"width: %d%%\" ></div>", 
                $state,
                $perc);
            }
          }
        }
        return $res;
  }

}

